@@ -10,7 +10,7 @@ use strict;
 
 use SWISS::TextFunc;
 
-$VERSION='1.79';
+$VERSION='1.80';
 
 # * Initialisation
 
@@ -274,7 +274,9 @@ sub toFastaOld {
 
 
 sub toFasta {
-    my $self          = shift;
+    my $self      = shift;
+    my $form_name = shift;
+
     my $FASTA_LINELEN = 60;
     my $result        = "";
 
@@ -285,8 +287,18 @@ sub toFasta {
         };
         return 0;
     }
+
+    my $ac = $form_name ? ($self->isoIds( $form_name ))[0] : $self->AC;
+    if ( !$ac ) {
+        if ($main::opt_warn && $form_name) {
+            carp "Unknown form $form_name; No Fasta written for $self";
+        };
+        return 0;
+    }
+
     # fasta header ">sp|AC|ID DE OS= OX= [GN= ] [PE= ] [SV=]"
     ( my $name = $self->DEs->head->text || '?' ) =~ s/ precursor$//;
+    $name = "Isoform $form_name of ".$name if ( $form_name );
       my $os   = $self->OSs->head->text || '?';
       my $ox   = $self->OXs->NCBI_TaxID()->head ? $self->OXs->NCBI_TaxID()->head->text || '?' : '?';
     ( my $organism = $os ) =~ s/ \(.+$//;
@@ -299,8 +311,8 @@ sub toFasta {
     ( my $pe = $self->PE->toText() ) =~ s/:.+$//;
     my   $sv = $self->DTs->SQ_version();
     $name  .= ' (' . $self->DEs->hasFragment . ')' if $self->DEs->hasFragment;
-    $result = '>' . ( $self->isCurated ? 'sp' : 'tr' ). '|' . 
-            $self->AC . '|' . $self->ID . ' ' . 
+    $result = '>' . ( $self->isCurated ? 'sp' : 'tr' ). '|' .
+            $ac . '|' . $self->ID . ' ' .
             $name . ' OS=' . $organism .
                     ' OX=' . $ox .
             ( $gn ? " GN=$gn" : '' ) .
@@ -308,13 +320,102 @@ sub toFasta {
             ( $sv ? " SV=$sv" : '' );
     $result .= "\n";
 
+    my $seq = $self->isoFormSequence( $form_name );
     # format the sequence, $FASTA_LINELEN AAs per line
-    $result .= join "\n", ( $self->SQ =~ m/.{1,$FASTA_LINELEN}/g );
+    $result .= join "\n", ( $seq =~ m/.{1,$FASTA_LINELEN}/g );
     $result .= "\n";
 
     return $result;  
 }
 
+
+sub isoFormSequence {
+    my $entry     = shift;
+    my $form_name = shift;
+
+    my $canonical = $entry->SQs->seq;
+    unless ( $form_name ) { # retrieve canonical sequence (default)
+       return $canonical
+    }
+    else { # isoform name specified: retrieve form sequence (rebuilt it from canonical + VAR_SEQ data)
+        my $ap = ( $entry->CCs->get( 'ALTERNATIVE PRODUCTS' ) )[0];
+        return undef unless $ap;
+
+        my %h_varseq  = ();
+        for my $varseq ( $entry->FTs->get( 'VAR_SEQ' ) ) { # preload data for all VAR_SEQs (by VAR_SEQ id)
+            my ( $_key, $_from, $_to, $_desc, $_qualifier, $id ) = @$varseq;
+            $h_varseq{ $id } = $varseq;
+        }
+        my $event = ($ap->getEventNames)[0]; # consider 1st event... sk api still wants event name even as form and event are not linked! (so in "model" all forms are "under" all events)
+        my @seqs = grep { !/^(External|Not described|Unknown)$/ } $ap->getFeatIds( $event, $form_name ); # list of "Sequence"=field associated to form (e.g. Displayed; VSP_000004, VSP_000009)
+        return( undef ) unless @seqs;
+        return( $canonical ) if $seqs[0] eq 'Displayed';
+        ## Rebuild form sequence based applying correspond VAR_SEQ modifications over canonical sequence
+        my $iso_seq = $canonical;
+        # prepare from modifications
+        my %h_modif = ();
+        for my $varseq_id ( @seqs ) { # for each associated VAR_SEQ id (VSP_) store VAR_SEQ data by from position key
+            my $varseq = $h_varseq{ $varseq_id };
+            my ( $_key, $from, $_to, $_desc, $_qualifier, $_id ) = @$varseq;
+            $from =~ s/<|\?//g;
+            $h_modif{ $from } = $varseq;
+        }
+        # sort the 'from' position in reverse numeric order to apply the VAR_SEQ modifications starting from the C-terminus.
+        for my $from ( reverse sort { $a <=> $b } keys %h_modif ) {
+            my $varseq = $h_modif{ $from };
+            my ( $_key, $from, $to, $desc, $_qualifier, $_id ) = @$varseq;
+            $from =~ s/<|\?//g;
+            $to   =~ s/>|\?//g;
+            if ( $desc =~ /^Missing/ ) { # deletion
+                $iso_seq = substr( $iso_seq, 0, $from - 1 ) . substr( $iso_seq, $to );
+            }
+            else { # substitution
+                $desc =~ s/[ \n]//g;
+                my ( $substitution ) = $desc =~ /\w+->(\w+)/;
+                $iso_seq = substr( $iso_seq, 0, $from - 1 ) . $substitution . substr( $iso_seq, $to );
+            }
+        }
+        return $iso_seq;
+    }
+}
+
+sub isoIds { # return isoform Id(s) for a particular (iso)form name (for canonical if no name provided)
+    my $entry     = shift;
+    my $form_name = shift;
+    my $ap        = ( $entry->CCs->get( 'ALTERNATIVE PRODUCTS' ) )[0];
+    return( undef ) unless defined $ap;
+
+    my $event = ($ap->getEventNames)[0]; # consider 1st event... sk api still wants event name even as form and event are not linked! (so in "model" all forms are "under" all events)
+    if ( $form_name ) {
+        return $ap->getIsoIds( $event, $form_name );
+    }
+    else { # from name not provided: look for canonical
+        for my $formName ( grep { defined $_ } $ap->getFormNames( $event ) ) {
+            my @seqs = $ap->getFeatIds( $event, $formName );
+            next unless $seqs[0] eq "Displayed";
+            return $ap->getIsoIds( $event, $formName );
+        }
+    }
+}
+sub isoId { return( ( isoIds( @_ ) )[0] ) } # return primary isoform Id for a particular (iso)form name (for canonical if no name provided)
+
+sub isoFormNames { # return isoform names (filtered for not: External|Displayed|Not described|Unknown) unless param all is true
+    my $entry  = shift;
+    my $all    = shift;
+    my $ap = ( $entry->CCs->get( 'ALTERNATIVE PRODUCTS' ) )[0];
+    return( undef ) unless defined $ap;
+
+    my @forms;
+    my $event = ($ap->getEventNames)[0]; # consider 1st event... sk api still wants event name even as form and event are not linked! (so in "model" all forms are "under" all events)
+    for my $formName ( grep { defined $_ } $ap->getFormNames( $event ) ) {
+        my @seqs = $ap->getFeatIds( $event, $formName ); # list of "Sequence"=field associated to form (e.g. Displayed; VSP_000004, VSP_000009)
+        next if !$all && $seqs[0] =~ /^(External|Displayed|Not described|Unknown)$/;
+        push @forms, $formName;
+    }
+    return( @forms );
+}
+
+
 # If this funtion returns true for an entry, the entry should be
 # processed correctly by swissknife. It does not mean that the entry
 # is syntactically correct. 
@@ -572,9 +673,30 @@ Return the entry in flat file text format. If internal comments and indentation
 have been removed as specified in the parameters to fromText(), you may wish
 to reinsert them in the text output by setting $insertInternalComments to true.
 
-=item toFasta
+=item toFasta [isoformName]
+
+Return the entry in Fasta format (canonical/displayed sequence if no isoform name provided)
+
+
+=item isoFormNames [all]
+
+Return the list of (filtered for not: External|Displayed|Not described|Unknown) isoform names.
+If all is true: list all isoform names
+
+
+=item isoIds [isoformName]
+
+Return list of isoform Ids for a particular isoform name (isoIds for canonical/displayed if no isoform name provided)
+
+=item isoId [isoformName]
+
+Return primary isoform Id for a particular isoform name (primary isoId for canonical/displayed if no isoform name provided)
+
+
+=item isoFormSequence [isoform name]
+
+Return the raw sequence for a specified isoform name (raw canonical sequence if no isoform name provided)
 
-Return the entry in Fasta format.
 
 =item equal
 
