@@ -382,18 +382,16 @@ sub getObject {
 
 # Evidence tag handling
 sub evidenceTagPosition { 
-# find index of evtag in an array "object" (e.g.a FT: [ key, from, to, text, evid!?, ftid, evidenceTags] evtag index is 6)
+# find index of evtag in an array "object" (e.g. a FT: [ key, from, to, text, qualifier, ftid, isoform, evidenceTags ] evtag index is 7)
   my ($arrayP) = @_;
   if ($#$arrayP == -1) {
     return 0;
   }
   # generaly is last element
-  if (@$arrayP[$#$arrayP] eq '{}') { return $#$arrayP; } 
-  elsif (@$arrayP[$#$arrayP] =~ $SWISS::TextFunc::evidencePattern) { # last element is evtag
+  if (@$arrayP[$#$arrayP] eq '{}') { return $#$arrayP; } # recognizable empty ev in last elem
+  elsif (@$arrayP[$#$arrayP] =~ $SWISS::TextFunc::evidencePattern) { # ev in last elem
     return $#$arrayP;
-  } else { 
-  	# not found in last element = array has yet no evtag
-  	# evtag should be added after end of array, index = 1 outside array (= array size)
+  } else { # not found in last element = array has yet no evtag, evtag should be added after end of array, index = 1 outside array (= array size)
     return $#$arrayP+1;
   }
 }
